@@ -296,7 +296,7 @@ runCtrbUrl ctrbfilename theHost =
 -- detailed description.
 fullmake :: String -> String -> IO ()
 fullmake dir s = do
-  _ <- system ("curl --retry-all-errors --retry 3 -s -w \"%{http_code} %{url} %{filename_effective}\\n\" --compressed -K " ++ (dir </> ("curlrun" ++ s)) ++ " --parallel -L > " ++ (dir </> ("curloutput" ++ s ++ ".1")))
+  _ <- system ("curl --parallel-max 2 --retry-all-errors --retry 3 -s -w \"%{http_code} %{url} %{filename_effective}\\n\" --compressed -K " ++ (dir </> ("curlrun" ++ s)) ++ " --parallel -L > " ++ (dir </> ("curloutput" ++ s ++ ".1")))
   mymake dir s 1
 
 mymake :: String -> String -> Integer -> IO ()
@@ -305,7 +305,7 @@ mymake dir s n = do
   let list = (filter ppred (map (splitOn " ") (splitOn "\n" text)))
   putStrLn ("Number of Failures " ++ (show (length list)))
   Tools.writeFile (dir </> ("curlrun" ++ s ++ "." ++ (show (n + 1)))) (concat ((map jjoin list) :: [String]))
-  _ <- if (list == []) then (return ExitSuccess) else system ("curl --retry-all-errors --retry 3 -s --compressed -K " ++ (dir </> ("curlrun" ++ s ++ "." ++ (show (n + 1)))) ++ " --parallel -L -w \"%{http_code} %{url} %{filename_effective}\\n\"  > " ++ (dir </> ("curloutput" ++ s ++ "." ++ (show (n + 1)))))
+  _ <- if (list == []) then (return ExitSuccess) else system ("curl --parallel-max 2 --retry-all-errors --retry 3 -s --compressed -K " ++ (dir </> ("curlrun" ++ s ++ "." ++ (show (n + 1)))) ++ " --parallel -L -w \"%{http_code} %{url} %{filename_effective}\\n\"  > " ++ (dir </> ("curloutput" ++ s ++ "." ++ (show (n + 1)))))
   if ((list == [])) then return () else (mymake dir s (n + 1))
   where
     ppred (x : _) | x == "429" = True
