@@ -39,11 +39,26 @@ import Tools
 import UrlAnalyse
 import WikiHelper
 
+-- | A parser to match the pairs of opening and closing brackets and the
+-- substring in between them and replace the three with an application of a
+-- function on the substring. See `runAction` for details. This parser matches
+-- only if to the string it is applied to is starting after the opening bracket.
 notendyet ::
+  -- | The function to be applied to the substring.
   (String -> ImperativeMonad String) ->
+  -- | The parser for the opening bracket.
   ParsecT String () ImperativeMonad String ->
+  -- | The parser for the closing bracket.
   ParsecT String () ImperativeMonad String ->
+  -- | The accumulator to accumulate characters between the opening and closing
+  -- bracket. Has to be the empty string when called externally.
   String ->
+  -- | The parser for matching all pairs of opening and closing brackets and the
+  -- substrings in between each of the pair and replaces each triple (opening
+  -- brackets, substring, closing bracket) with the application of function
+  -- given in the first parameter on the substring.  See `runAction` for
+  -- details. This parser only match if the string it is applied to starts
+  -- immediately after the opening bracket.
   ParsecT String () ImperativeMonad String
 notendyet action sstart eend aku =
   try
@@ -62,10 +77,24 @@ notendyet action sstart eend aku =
       a <- anyChar
       notendyet action sstart eend (aku ++ [a])
 
+-- | A parser to match the pairs of opening and closing brackets and the
+-- substring in between them and replace the three with an application of a
+-- function on the substring. See `runAction` for details. This parser matches
+-- only if the string it is applied to begins with an opening bracket or is the
+-- empty string itself.
 beginning ::
+  -- | The function to be applied to the substring.
   (String -> ImperativeMonad String) ->
+  -- | The parser for the opening bracket.
   ParsecT String () ImperativeMonad String ->
+  -- | The parser for the closing bracket.
   ParsecT String () ImperativeMonad String ->
+  -- | The parser for matching all pairs of opening and closing brackets and the
+  -- substrings in between each of the pair and replaces each triple (opening
+  -- brackets, substring, closing bracket) with the application of function
+  -- given in the first parameter on the substring.  See `runAction` for
+  -- details. This parser only match if the string it is applied to is the empty
+  -- string or starts with and opening bracket.
   ParsecT String () ImperativeMonad [Char]
 beginning action sstart eend =
   try
@@ -78,10 +107,21 @@ beginning action sstart eend =
       ne <- notendyet action sstart eend []
       return (ne)
 
+-- | A parser to match the pairs of opening and closing brackets and the
+-- substring in between them and replace the three with an application of a
+-- function on the substring. See `runAction` for details.
 startToEnd ::
+  -- | The function to be applied to the substring.
   (String -> ImperativeMonad String) ->
+  -- | The parser for the opening bracket.
   ParsecT String () ImperativeMonad String ->
+  -- | The parser for the closing bracket.
   ParsecT String () ImperativeMonad String ->
+  -- | The parser for matching all pairs of opening and closing brackets and the
+  -- substrings in between each of the pair and replaces each triple (opening
+  -- brackets, substring, closing bracket) with the application of function
+  -- given in the first parameter on the substring.  See `runAction` for
+  -- details.
   ParsecT String () ImperativeMonad String
 startToEnd action sstart eend =
   try
@@ -95,14 +135,27 @@ startToEnd action sstart eend =
       s <- startToEnd action sstart eend
       return (a : s)
 
-zeroAction :: (Monad m) => t -> t1 -> m [Char]
+-- | Action to replace the brackets and the substring in between with the empty
+-- string (see `runAction` for details).
+zeroAction :: WikiUrl -> String -> ImperativeMonad String
 zeroAction _ _ = return ""
 
+-- | Looks at a string and does some replacement in it. It looks for opening and
+-- closing brackets (given in the first and second parameter) and the substrings
+-- between each pair of opening and closing brackets. A function given in the
+-- third parameter is applied to the each substring and the each pair of
+-- brackets including the substring between them is replaced by the result of
+-- the application of the function given in the third parameter.
 runAction ::
+  -- | The opening bracket.
   String ->
+  -- | The closing bracket.
   String ->
+  -- | The function to be applied.
   (String -> ImperativeMonad String) ->
+  -- | The input string to process.
   String ->
+  -- | The resulting string wrapped in the ImperativeMonad
   ImperativeMonad String
 runAction sstart eend action text =
   do
@@ -117,7 +170,19 @@ runAction sstart eend action text =
       Left _ -> return ""
       Right xs -> return xs
 
-chapterAction :: WikiUrl -> String -> ImperativeMonad String
+-- | A function to download the content of a linked sub website and replace the
+-- link with the downloaded content. See also `runAction`
+chapterAction ::
+  -- | The `WikiUrl` the main website of which the sub website shall be
+  -- downloaded
+  WikiUrl ->
+  -- | The contents of the wiki template which requests the inclusion of the sub
+  -- website
+  String ->
+  -- | ImperativeMonad action downloading the sub website and returning it. Also
+  -- the contributors of the sub website are accumulated in the state of the
+  -- ImperativeMonad.
+  ImperativeMonad String
 chapterAction wurl text =
   do
     pp <- liftIO (getpage d (wurl))
@@ -129,7 +194,22 @@ chapterAction wurl text =
   where
     d = (trim (takeWhile (/= '|') text))
 
-chapterAction2 :: WikiUrl -> String -> String -> ImperativeMonad String
+-- | A function to download the content of a linked sub website and replace the
+-- link with the downloaded content. See also `runAction`
+chapterAction2 ::
+  -- | The `WikiUrl` the main website of which the sub website shall be
+  -- downloaded
+  WikiUrl ->
+  -- | The lemma of the main website of which the sub website shall be
+  -- downloaded
+  String ->
+  -- | The contents of the wiki template which requests the inclusion of the sub
+  -- website
+  String ->
+  -- | ImperativeMonad action downloading the sub website and returning it. Also
+  -- the contributors of the sub website are accumulated in the state of the
+  -- ImperativeMonad.
+  ImperativeMonad String
 chapterAction2 wurl lema text =
   do
     pp <- liftIO (getpage d (wurl))
@@ -144,25 +224,19 @@ chapterAction2 wurl lema text =
     e = (trim (takeWhile (/= '|') text))
     d = (removePrintVersion lema) ++ "/" ++ e
 
-chapterAction3 :: WikiUrl -> String -> String -> ImperativeMonad String
-chapterAction3 wurl text lema =
-  do
-    pp <- liftIO (getpage d (wurl))
-    case pp of
-      Just p -> do
-        _ <- addContributors d Nothing
-        noinclude
-          wurl
-          ("\n\ndhunparserurl " ++ d ++ "\n\n" ++ "= " ++ e ++ " =\n" ++ p)
-      _ -> return ""
-  where
-    e = (trim (takeWhile (/= '|') text))
-    d = (removePrintVersion lema) ++ "/ " ++ e
-
-includeAction :: WikiUrl -> String -> ImperativeMonad String
-includeAction = qIncludeAction
-
-qIncludeAction :: WikiUrl -> String -> ImperativeMonad String
+-- | A function to download the content of a linked sub website and replace the
+-- link with the downloaded content. See also `runAction`
+qIncludeAction ::
+  -- | The `WikiUrl` the main website of which the sub website shall be
+  -- downloaded
+  WikiUrl ->
+  -- | The contents of the wiki template which requests the inclusion of the sub
+  -- website
+  String ->
+  -- | ImperativeMonad action downloading the sub website and returning it. Also
+  -- the contributors of the sub website are accumulated in the state of the
+  -- ImperativeMonad.
+  ImperativeMonad String
 qIncludeAction wurl text =
   if isInfixOf "Vorlage" text
     then return ("{{" ++ text ++ "}}")
@@ -181,7 +255,16 @@ qIncludeAction wurl text =
   where
     d = (trim (takeWhile (/= '|') text))
 
-runqBookIncludeAction :: String -> ImperativeMonad ()
+-- | This function downloads the wiki source or HTML form the wiki server and
+-- parses it to a parse tree. It writes the parse tree to disk. Also writes the
+-- accumulated filenames of this storage files together with the the accumulated
+-- contributor information to disk.
+runqBookIncludeAction ::
+  -- | The directory where to read the parameters file from and store the result
+  -- to.
+  String ->
+  -- | The action as described above.
+  ImperativeMonad ()
 runqBookIncludeAction dir =
   do
     t <- liftIO $ B.readFile (dir </> "bookinput")
@@ -192,7 +275,21 @@ runqBookIncludeAction dir =
     st <- get
     liftIO $ B.writeFile (dir </> "bookoutput") (S.encode (loadacu st, (audict st) ++ ad))
 
-qBookIncludeActionbase :: FullConfig -> WikiUrl -> String -> ImperativeMonad String
+-- | This function downloads the wiki source or HTML form the wiki server and
+-- parses it to a parse tree. It writes the parse tree to disk and accumulates
+-- the name of the storage file in the loadacu. It also adds the contributors of
+-- the current article to the list of contributors.
+qBookIncludeActionbase ::
+  -- | The Configuration the current run of mediawiki2latex .
+  FullConfig ->
+  -- | The `WikiUrl` where the main page of the book is stored .
+  WikiUrl ->
+  -- | The name of the sub page of the book to be processed (the lemma).
+  String ->
+  -- | Run the action as explained above and returns the wiki source code or
+  -- HTML code downloaded with an added marker which is used to identify the sub
+  -- page when parsing the wiki text.
+  ImperativeMonad String
 qBookIncludeActionbase cfg wurl text =
   if isInfixOf "Vorlage" text
     then return ("{{" ++ text ++ "}}")
@@ -231,7 +328,26 @@ qBookIncludeActionbase cfg wurl text =
       (ExpandedTemplates _) -> (loadMediaWiki d wurl) >>= (return . Just)
       _ -> (loadPlain d wurl Nothing) >>= (return . Just)
 
-qBookIncludeAction :: FullConfig -> WikiUrl -> String -> ImperativeMonad String
+-- | This action downloads the wiki source or HTML from the MediaWiki server and
+-- reads it into a parse tree. The operation of this action depends on the
+-- selected output format. In case the output format is LaTeX or PDF (loadacu is
+-- in Left case) it loads an parses each article separately and stores the parse
+-- tree on disk and accumulates the filenames of the storage files in the
+-- loadacu. In all other cases the parse trees will be accumulated in the
+-- loadacu. It spawns sub processes for each article to be processed in order to
+-- save memory since the memory of each subprocess is freed upon its
+-- termination.
+qBookIncludeAction ::
+  -- | The configuration the current run of mediawiki2latex
+  FullConfig ->
+  -- | The `WikiUrl` of the books main page (the page that contains the links to
+  -- the actual articles in the book)
+  WikiUrl ->
+  -- | The lemma (names of the article) of the current sub page of the book
+  String ->
+  -- | Run the action causing the side effects described above and return the
+  -- wiki source or HTML loaded from the wiki server.
+  ImperativeMonad String
 qBookIncludeAction cfg wurl text =
   do
     sst <- get
@@ -289,59 +405,6 @@ qBookIncludeAction cfg wurl text =
       (ExpandedTemplates _) -> (loadMediaWiki d wurl) >>= (return . Just)
       _ -> (loadPlain d wurl Nothing) >>= (return . Just)
 
-makeUrl :: String -> String -> String -> [Char]
-makeUrl llang theFam thePage =
-  (unify . exportURL)
-    ( if isInfixOf "commons" llang
-        then
-          ( URL
-              { url_path = "~daniel/WikiSense/Contributors.php",
-                url_params =
-                  [ ("wikifam", "commons.wikimedia.org"),
-                    ("page", thePage),
-                    ("since", ""),
-                    ("until", ""),
-                    ("grouped", "on"),
-                    ("hideanons", "on"),
-                    ("max", "100000"),
-                    ("format", "html")
-                  ],
-                url_type =
-                  Absolute
-                    ( Host
-                        { protocol = HTTP True,
-                          host = "toolserver.org",
-                          port = Nothing
-                        }
-                    )
-              }
-          )
-        else
-          ( URL
-              { url_path = "~daniel/WikiSense/Contributors.php",
-                url_params =
-                  [ ("wikilang", llang),
-                    ("wikifam", theFam),
-                    ("page", thePage),
-                    ("since", ""),
-                    ("until", ""),
-                    ("grouped", "on"),
-                    ("hideanons", "on"),
-                    ("max", "100000"),
-                    ("format", "html")
-                  ],
-                url_type =
-                  Absolute
-                    ( Host
-                        { protocol = HTTP True,
-                          host = "toolserver.org",
-                          port = Nothing
-                        }
-                    )
-              }
-          )
-    )
-
 langau :: Map String String
 langau =
   fromList
@@ -540,7 +603,16 @@ addContributors theLemma uu =
         Nothing -> Just Contributor {name = a, href = h, edits = e}
         Just old -> Just old {edits = (edits old) + e}
 
-noinclude :: t -> String -> ImperativeMonad [Char]
+-- | replace all content between noinclude tags with empty string including the
+-- tags itself.
+noinclude ::
+  -- | The WikiUrl.
+  WikiUrl ->
+  -- | The string to in which to remove the noinclude tags.
+  String ->
+  -- | The input string with the noinclude tags removed wrapped into
+  -- the`ImperativeMonad`
+  ImperativeMonad String
 noinclude wurl =
   runAction "<noinclude>" "</noinclude>" (zeroAction wurl)
 
@@ -548,11 +620,10 @@ runActions :: WikiUrl -> String -> String -> ImperativeMonad String
 runActions wurl lema text =
   do
     x <- noinclude wurl text
-    y <- runAction "{{Druckversion Kapitel|" "}}" (chapterAction3 wurl lema) x
-    v <- runAction "{{Print entry|" "}}" (chapterAction2 wurl lema) y
+    v <- runAction "{{Print entry|" "}}" (chapterAction2 wurl lema) x
     z <- runAction "{{print entry|" "}}" (chapterAction2 wurl lema) v
     a <- runAction "{{Print entry|" "}}" (chapterAction wurl) z
-    b <- runAction "{{:" "}}" (includeAction wurl) a
+    b <- runAction "{{:" "}}" (qIncludeAction wurl) a
     c <- runAction "{{:" "}}" (qIncludeAction wurl) b
     d <- runAction "{{:" "}}" (qIncludeAction wurl) c
     e <- runAction "{{:" "}}" (qIncludeAction wurl) d
@@ -564,7 +635,20 @@ runActions wurl lema text =
     k <- runAction "{{:" "}}" (qIncludeAction wurl) j
     runAction "{{:" "}}" (qIncludeAction wurl) k
 
-runBookActions :: FullWikiUrl -> String -> FullConfig -> ImperativeMonad String
+-- | Run the actions to download and compile the sub pages of a book when
+-- mediawiki2latex is running in book mode. This will involve calls of the
+-- mediawiki2latex executable to the mediawiki2latex executable itself. This is
+-- done in order to save memory since the memory of a subprocess will be freed
+-- upon its termination.
+runBookActions ::
+  -- | The FullWikiUrl of the collection page of the book.
+  FullWikiUrl ->
+  -- | The contents of the collection page as wiki source text.
+  String ->
+  -- | The configuration for this run of mediawiki2latex.
+  FullConfig ->
+  -- | The IO action loading the sub pages of the book.
+  ImperativeMonad String
 runBookActions fu text cfg =
   do
     x <- noinclude wurl text
@@ -601,7 +685,22 @@ loadPlain lema wurl uu =
         runActions wurl lema p
       _ -> throwError (DownloadError lema (show wurl))
 
-loadBook :: ImperativeState -> Maybe URL -> FullConfig -> ImperativeMonad [Char]
+-- | Loads a book from the wiki. A book is a collection of wiki links to lemmas.
+-- If the selected output format is either PDF or LaTeX source this will also
+-- include the reading of the input code (wiki or HTML) to a parse tree and the
+-- storage of this parse tree on disk. This is done in several subprocess in
+-- order to save memory since the memory of each subprocess is freed upon its
+-- termination.
+loadBook ::
+  -- | The state of the imperative monad on the start of this action.
+  ImperativeState ->
+  -- | The URL under which the collection is located.
+  Maybe URL ->
+  -- | The configuration for the run of mediawiki2latex.
+  FullConfig ->
+  -- | The action downloading the book and adding the contributor information to
+  -- the state of the ImperativeMonad.
+  ImperativeMonad [Char]
 loadBook st uu cfg =
   let fu = fullUrl st
    in do
@@ -626,28 +725,18 @@ loadHTML st =
         x <- liftIO (geturl2 (exportURL (url fu)))
         return ("\n\ndhunparserurl " ++ (lemma (fullUrl st)) ++ "\n\n" ++ ((C.decode . unpack) x))
 
-loadHTMLnoQuote :: ImperativeState -> ImperativeMonad String
-loadHTMLnoQuote st =
-  let fu = fullUrl st
-   in do
-        x <- liftIO (geturl2 (exportURL (url fu)))
-        return . C.decode . unpack $ x
-
-loadBookHTML :: ImperativeState -> ImperativeMonad String
-loadBookHTML st =
-  let fu = fullUrl st
-   in do
-        midst <- get
-        (res, newst) <-
-          liftIO
-            (runStateT (runExceptT (loadPlain (lemma (fullUrl st)) (wikiUrl (fullUrl st)) (Just (url fu)))) midst)
-        case res of
-          Right _ -> put newst
-          _ -> return ()
-        x <- liftIO (geturl2 (exportURL (url fu)))
-        return . C.decode . unpack $ x
-
-loadMediaWiki :: String -> WikiUrl -> ImperativeMonad [Char]
+-- | Load the wiki code with the templates expanded by MediaWiki from a
+-- MediaWiki server.
+loadMediaWiki ::
+  -- | The lemma on the wiki to be downloaded
+  String ->
+  -- | The WikiUrl of the lemma to be downloaded (see type `UrlAnalyse.WikiUrl`
+  -- for details)
+  WikiUrl ->
+  -- | The wiki source text with the templates expanded wrapped into the
+  -- `ImperativeMonad` since the download requires an IO action and can possibly
+  -- fail.
+  ImperativeMonad [Char]
 loadMediaWiki lema wurl =
   do
     pp <- liftIO (getpage2 lema wurl)
@@ -688,10 +777,10 @@ loadMediaWiki lema wurl =
               (DownloadError lema (show wurl))
       _ -> throwError (DownloadError lema (show wurl))
 
--- | main function to download images form the wiki. It takes the RunMode as
--- only parameter. In case of HTML the html from the website is loaded. In all
+-- | main function to download content form the wiki. It takes the RunMode as
+-- only parameter. In case of HTML the HTML from the website is loaded. In all
 -- other cases the wiki source text is downloaded. In case of ExpandedTemplates
--- the templates are also expanded by mediawiki running on the wiki website
+-- the templates are also expanded by MediaWiki running on the wiki website
 load :: FullConfig -> ImperativeMonad String
 load cfg =
   do
